﻿using System.Windows;
using System.ServiceModel;
using System.Diagnostics;
using System;
using System.Windows.Media;
using System.IO;
using Microsoft.Win32;
using WCF_ClientApp.ServiceReference1;

namespace WCF_ClientApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        VCServiceClient _Client;
        CallbackService _CallbackService;
        Process _Process;

        bool IsSubScribed;
        bool VCStarted;
        public MainWindow()
        {
            InitializeComponent();

            StartVC.Click += StartVC_Click;
            ConnectSubscribeToVCCallbacks.Click += ConnectSubscribeToVCCallbacks_Click;
            LoadedComponents.SelectionChanged += LoadedComponents_SelectionChanged;
            LoadFile.Click += LoadFile_Click;
            _Client = null;
            _CallbackService = null;
            _Process = null;
            IsSubScribed = false;
            VCStarted = false;
            StartVC.Background = Brushes.LightGray;
            ConnectSubscribeToVCCallbacks.Background = Brushes.LightGray;
            LoadFile.Background = Brushes.LightGray;
            Signal.Background = Brushes.LightGray;
            //Signal.FontSize = 22;
            Signal.Content = "No Signal";
            ConnectSubscribeToVCCallbacks.Content = "Connect to VC 4.0";
            StartVC.Content = "Start VC 4.0";

            this.Activated += MainWindow_Activated;
            this.Deactivated += MainWindow_Deactivated;
        }

        private void MainWindow_Deactivated(object sender, EventArgs e)
        {
            this.Topmost = true;
            this.Activate();
        }

        private void MainWindow_Activated(object sender, EventArgs e)
        {
            this.Topmost = true;
        }

        private void LoadedComponents_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (LoadedComponents.SelectedValue != null)
            {
                _Client.SelectComponent(LoadedComponents.SelectedValue.ToString());
            }
        }

        private void LoadFile_Click(object sender, RoutedEventArgs e)
        {
            Stream _myStream = null;
            OpenFileDialog _openFileDialog = new OpenFileDialog();
            _openFileDialog.InitialDirectory = "C:\\Temp\\VC_WCF";
            _openFileDialog.Filter = "VC 4.0 files (*.vcmx)|*.vcmx|All files (*.*)|*.*";
            _openFileDialog.FilterIndex = 1;
            _openFileDialog.RestoreDirectory = true;

            if (_openFileDialog.ShowDialog() == true)
            {
                try
                {
                    if ((_myStream = _openFileDialog.OpenFile()) != null)
                    {
                        using (_myStream)
                        {
                            _Client.LoadLayout(_openFileDialog.FileName);
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: Could not read file from disk. Original error: " + ex.Message);
                }
            }
        }

        private void ConnectSubscribeToVCCallbacks_Click(object sender, RoutedEventArgs e)
        {
            ConnectSubscribeToVCWcf();
        }

        private void StartVC_Click(object sender, RoutedEventArgs e)
        {
            startVC();
        }

        private void ConnectSubscribeToVCWcf()
        {
            try
            {
                if (VCStarted == true)
                {
                    if (IsSubScribed == false)
                    {
                        _CallbackService = new CallbackService(this);
                        _Client = new VCServiceClient(new InstanceContext(_CallbackService));
                        _Client.Subscribe();
                        ConnectSubscribeToVCCallbacks.Background = Brushes.LightGreen;
                        IsSubScribed = true;
                        ConnectSubscribeToVCCallbacks.Content = "UnConnect from VC 4.0";
                        return;
                    }
                    if (IsSubScribed == true)
                    {
                        _CallbackService = null;
                        _Client.UnSubscribe();
                        _Client.Abort();
                        IsSubScribed = false;
                        ConnectSubscribeToVCCallbacks.Background = Brushes.LightGray;
                        ConnectSubscribeToVCCallbacks.Content = "Connect to VC 4.0";
                        Signal.Content = "No Signal";
                        Signal.Background = Brushes.LightGray;
                        LoadedComponents.Items.Clear();
                        return;
                    }
                }
            }
            catch
            {
                MessageBox.Show("D'oh! VC 4.0 WCF service is not ready");
            }   
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (_Process != null)
            {
                _Process.Kill();
            }
        }

        private void startVC()
        {
            if (VCStarted == false)
            {
                try
                {
                    // 1. Start application
                    string address = "http://+:8000/vcwcfdemo/service";
                    string args = string.Format(@"http add urlacl url={0} user={1}\{2}", address, Environment.UserDomainName, Environment.UserName);
                    ProcessStartInfo startInfo = new ProcessStartInfo("netsh", args);
                    string path = Directory.GetCurrentDirectory();
                    Console.WriteLine(path);
                    startInfo.FileName = @"C:\Temp\VC_WCF\WcfDemo\Output\AnyCPU\VisualComponents.Engine.exe";
                    startInfo.Verb = "runas";
                    _Process = new Process();
                    _Process.StartInfo = startInfo;
                    _Process.EnableRaisingEvents = true;
                    _Process.Exited += _Process_Exited;
                    _Process.Start();
                    StartVC.Background = Brushes.LightGreen;
                    VCStarted = true;
                    StartVC.Content = "Exit VC 4.0";
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: Could not read file from disk. Original error: " + ex.Message);
                }
            }
            else
            {
                if (IsSubScribed == true)
                {
                    ConnectSubscribeToVCWcf();
                }
                _Process.Kill();
                StartVC.Background = Brushes.LightGray;
                StartVC.Content = "Start VC 4.0";
                LoadedComponents.Items.Clear();
                Signal.Background = Brushes.LightGray;
                Signal.Content = "No Signal";
                VCStarted = false;
            }
        }

        private void _Process_Exited(object sender, EventArgs e)
        {
            _Process = null;
        }
    }
}
